@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005-2007 Sun Microsystems, Inc.
-// Copyright (C)2009-2012, 2014, 2017-2019 D. R. Commander
+// Copyright (C)2009-2012, 2014, 2017-2020 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -54,12 +54,9 @@ Frame::~Frame(void)
 
 void Frame::deInit(void)
 {
-	if(bits && primary)
+	if(primary)
 	{
 		delete [] bits;  bits = NULL;
-	}
-	if(rbits && primary)
-	{
 		delete [] rbits;  rbits = NULL;
 	}
 }
@@ -77,25 +74,21 @@ void Frame::init(rrframeheader &h, int pixelFormat, int flags_, bool stereo_)
 	if(h.framew != hdr.framew || h.frameh != hdr.frameh
 		|| newpf->size != pf->size || !bits)
 	{
-		if(bits) delete [] bits;
-		NEWCHECK(bits = new unsigned char[h.framew * h.frameh * newpf->size + 1]);
+		delete [] bits;
+		bits = new unsigned char[h.framew * h.frameh * newpf->size + 1];
 	}
 	if(stereo_)
 	{
 		if(h.framew != hdr.framew || h.frameh != hdr.frameh
 			|| newpf->size != pf->size || !rbits)
 		{
-			if(rbits) delete [] rbits;
-			NEWCHECK(rbits =
-				new unsigned char[h.framew * h.frameh * newpf->size + 1]);
+			delete [] rbits;
+			rbits = new unsigned char[h.framew * h.frameh * newpf->size + 1];
 		}
 	}
 	else
 	{
-		if(rbits)
-		{
-			delete [] rbits;  rbits = NULL;
-		}
+		delete [] rbits;  rbits = NULL;
 	}
 	pf = newpf;  pitch = pf->size * h.framew;  stereo = stereo_;  hdr = h;
 }
@@ -130,7 +123,7 @@ Frame *Frame::getTile(int x, int y, int width, int height)
 		|| (y + height) > hdr.height)
 		throw Error("Frame::getTile", "Argument out of range");
 
-	NEWCHECK(f = new Frame(false));
+	f = new Frame(false);
 	f->hdr = hdr;
 	f->hdr.x = x;
 	f->hdr.y = y;
@@ -524,27 +517,24 @@ void CompressedFrame::init(rrframeheader &h, int buffer)
 		case RR_LEFT:
 			if(h.width != hdr.width || h.height != hdr.height || !bits)
 			{
-				if(bits) delete [] bits;
-				NEWCHECK(bits = new unsigned char[tjBufSize(h.width, h.height,
-					h.subsamp)]);
+				delete [] bits;
+				bits = new unsigned char[tjBufSize(h.width, h.height, h.subsamp)];
 			}
 			hdr = h;  hdr.flags = RR_LEFT;  stereo = true;
 			break;
 		case RR_RIGHT:
 			if(h.width != rhdr.width || h.height != rhdr.height || !rbits)
 			{
-				if(rbits) delete [] rbits;
-				NEWCHECK(rbits = new unsigned char[tjBufSize(h.width, h.height,
-					h.subsamp)]);
+				delete [] rbits;
+				rbits = new unsigned char[tjBufSize(h.width, h.height, h.subsamp)];
 			}
 			rhdr = h;  rhdr.flags = RR_RIGHT;  stereo = true;
 			break;
 		default:
 			if(h.width != hdr.width || h.height != hdr.height || !bits)
 			{
-				if(bits) delete [] bits;
-				NEWCHECK(bits = new unsigned char[tjBufSize(h.width, h.height,
-					h.subsamp)]);
+				delete [] bits;
+				bits = new unsigned char[tjBufSize(h.width, h.height, h.subsamp)];
 			}
 			hdr = h;  hdr.flags = 0;  stereo = false;
 			break;
@@ -560,6 +550,9 @@ void CompressedFrame::init(rrframeheader &h, int buffer)
 
 // Frame created from shared graphics memory
 
+CriticalSection FBXFrame::mutex;
+
+
 FBXFrame::FBXFrame(Display *dpy, Drawable draw, Visual *vis,
 	bool reuseConn_) : Frame()
 {
@@ -583,6 +576,7 @@ void FBXFrame::init(char *dpystring, Drawable draw, Visual *vis)
 	memset(&fb, 0, sizeof(fbx_struct));
 
 	if(!dpystring || !draw) throw(Error("FBXFrame::init", "Invalid argument"));
+	CriticalSection::SafeLock l(mutex);
 	if(!(wh.dpy = XOpenDisplay(dpystring)))
 		throw(Error("FBXFrame::init", "Could not open display"));
 
@@ -618,10 +612,14 @@ void FBXFrame::init(rrframeheader &h)
 	if((env = getenv("VGL_USEXSHM")) != NULL && strlen(env) > 0
 		&& !strcmp(env, "0"))
 		usexshm = 0;
-	TRY_FBX(fbx_init(&fb, wh, h.framew, h.frameh, usexshm));
+	{
+		CriticalSection::SafeLock l(mutex);
+		TRY_FBX(fbx_init(&fb, wh, h.framew, h.frameh, usexshm));
+	}
 	if(h.framew > fb.width || h.frameh > fb.height)
 	{
 		XSync(wh.dpy, False);
+		CriticalSection::SafeLock l(mutex);
 		TRY_FBX(fbx_init(&fb, wh, h.framew, h.frameh, usexshm));
 	}
 	hdr = h;
@@ -678,6 +676,9 @@ void FBXFrame::redraw(void)
 
 // Frame created using X Video
 
+CriticalSection XVFrame::mutex;
+
+
 XVFrame::XVFrame(Display *dpy_, Window win_) : Frame()
 {
 	if(!dpy_ || !win_) throw(Error("XVFrame::XVFrame", "Invalid argument"));
@@ -699,6 +700,7 @@ void XVFrame::init(char *dpystring, Window win_)
 	memset(&fb, 0, sizeof(fbxv_struct));
 
 	if(!dpystring || !win_) throw(Error("XVFrame::init", "Invalid argument"));
+	CriticalSection::SafeLock l(mutex);
 	if(!(dpy = XOpenDisplay(dpystring)))
 		throw(Error("XVFrame::init", "Could not open display"));
 
@@ -741,10 +743,14 @@ XVFrame &XVFrame::operator= (Frame &f)
 void XVFrame::init(rrframeheader &h)
 {
 	checkHeader(h);
-	TRY_FBXV(fbxv_init(&fb, dpy, win, h.framew, h.frameh, I420_PLANAR, 0));
+	{
+		CriticalSection::SafeLock l(mutex);
+		TRY_FBXV(fbxv_init(&fb, dpy, win, h.framew, h.frameh, I420_PLANAR, 0));
+	}
 	if(h.framew > fb.xvi->width || h.frameh > fb.xvi->height)
 	{
 		XSync(dpy, False);
+		CriticalSection::SafeLock l(mutex);
 		TRY_FBXV(fbxv_init(&fb, dpy, win, h.framew, h.frameh, I420_PLANAR, 0));
 	}
 	hdr = h;
