@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005 Sun Microsystems, Inc.
-// Copyright (C)2011, 2014, 2017-2019 D. R. Commander
+// Copyright (C)2011, 2014, 2017-2020 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -19,6 +19,9 @@
 #include "Timer.h"
 #include "bmp.h"
 #include "vgllogo.h"
+#ifdef USEHELGRIND
+	#include <valgrind/helgrind.h>
+#endif
 
 using namespace vglutil;
 using namespace vglcommon;
@@ -61,14 +64,17 @@ class Blitter : public Runnable
 		{
 			for(int i = 0; i < NFRAMES; i++)
 			{
-				if(useGL) { NEWCHECK(frames[i] = new GLFrame(dpy, win)); }
+				if(useGL) { frames[i] = new GLFrame(dpy, win); }
 				#ifdef USEXV
-				else if(useXV) { NEWCHECK(frames[i] = new XVFrame(dpy, win)); }
+				else if(useXV) { frames[i] = new XVFrame(dpy, win); }
 				#endif
-				else { NEWCHECK(frames[i] = new FBXFrame(dpy, win)); }
+				else { frames[i] = new FBXFrame(dpy, win); }
 			}
-			NEWCHECK(thread = new Thread(this));
+			thread = new Thread(this);
 			thread->start();
+			#ifdef USEHELGRIND
+			ANNOTATE_BENIGN_RACE_SIZED(&deadYet, sizeof(bool), );
+			#endif
 		}
 
 		virtual ~Blitter(void)
@@ -149,7 +155,7 @@ class Blitter : public Runnable
 				fprintf(stderr, "Average Blitter performance = %f Mpixels/sec%s\n",
 					mpixels / totalTime, check ? " [PASSED]" : "");
 			}
-			catch(Error &e)
+			catch(std::exception &e)
 			{
 				if(thread) thread->setError(e);
 				for(int i = 0; i < NFRAMES; i++)
@@ -215,8 +221,11 @@ class Decompressor : public Runnable
 			blitter(blitter_), findex(0), deadYet(false), dpy(dpy_), win(win_),
 			myID(myID_), thread(NULL)
 		{
-			NEWCHECK(thread = new Thread(this));
+			thread = new Thread(this);
 			thread->start();
+			#ifdef USEHELGRIND
+			ANNOTATE_BENIGN_RACE_SIZED(&deadYet, sizeof(bool), );
+			#endif
 		}
 
 		virtual ~Decompressor(void) { shutdown(); }
@@ -275,7 +284,7 @@ class Decompressor : public Runnable
 					cframe.signalComplete();
 				}
 			}
-			catch(Error &e)
+			catch(std::exception &e)
 			{
 				if(thread) thread->setError(e);
 				for(int i = 0; i < NFRAMES; i++) cframes[i].signalComplete();
@@ -297,11 +306,16 @@ class Compressor : public Runnable
 	public:
 
 		Compressor(Decompressor *decompressor_, Blitter *blitter_) : findex(0),
-			deadYet(false), thread(NULL), decompressor(decompressor_),
-			blitter(blitter_)
+			deadYet(false), thread(NULL), decompressor(decompressor_)
+			#ifdef USEXV
+			, blitter(blitter_)
+			#endif
 		{
-			NEWCHECK(thread = new Thread(this));
+			thread = new Thread(this);
 			thread->start();
+			#ifdef USEHELGRIND
+			ANNOTATE_BENIGN_RACE_SIZED(&deadYet, sizeof(bool), );
+			#endif
 		}
 
 		virtual ~Compressor(void)
@@ -378,7 +392,7 @@ class Compressor : public Runnable
 					frame.signalComplete();
 				}
 			}
-			catch(Error &e)
+			catch(std::exception &e)
 			{
 				if(thread) thread->setError(e);
 				for(int i = 0; i < NFRAMES; i++) frames[i].signalComplete();
@@ -391,7 +405,9 @@ class Compressor : public Runnable
 		Thread *thread;
 		Frame frames[NFRAMES];
 		Decompressor *decompressor;
+		#ifdef USEXV
 		Blitter *blitter;
+		#endif
 };
 
 
@@ -408,10 +424,10 @@ class FrameTest
 				BlackPixel(dpy, DefaultScreen(dpy))));
 			ERRIFNOT(XMapRaised(dpy, win));
 
-			NEWCHECK(blitter = new Blitter(dpy, win, myID));
+			blitter = new Blitter(dpy, win, myID);
 			if(!useXV)
-				NEWCHECK(decompressor = new Decompressor(blitter, dpy, win, myID));
-			NEWCHECK(compressor = new Compressor(decompressor, blitter));
+				decompressor = new Decompressor(blitter, dpy, win, myID);
+			compressor = new Compressor(decompressor, blitter);
 		}
 
 		~FrameTest(void) { shutdown();  XDestroyWindow(dpy, win); }
@@ -500,12 +516,12 @@ class FrameTest
 		void shutdown(void)
 		{
 			fprintf(stderr, "Shutting down....\n");  fflush(stderr);
-			if(blitter) blitter->shutdown();
-			if(decompressor) decompressor->shutdown();
 			if(compressor) compressor->shutdown();
-			delete blitter;  blitter = NULL;
-			delete decompressor;  decompressor = NULL;
+			if(decompressor) decompressor->shutdown();
+			if(blitter) blitter->shutdown();
 			delete compressor;  compressor = NULL;
+			delete decompressor;  decompressor = NULL;
+			delete blitter;  blitter = NULL;
 		}
 
 		Display *dpy;  Window win;
@@ -660,7 +676,7 @@ int main(int argc, char **argv)
 
 			for(i = 0; i < NUMWIN; i++)
 			{
-				NEWCHECK(test[i] = new FrameTest(dpy, i));
+				test[i] = new FrameTest(dpy, i);
 			}
 
 			for(w = MINW; w <= MAXW; w += 33)
@@ -706,9 +722,9 @@ int main(int argc, char **argv)
 			fprintf(stderr, "\n");
 		}
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n%s\n", e.getMethod(), e.getMessage());
+		fprintf(stderr, "%s\n%s\n", GET_METHOD(e), e.what());
 		if(dpy) XCloseDisplay(dpy);
 		exit(1);
 	}
