@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005, 2006 Sun Microsystems, Inc.
-// Copyright (C)2009-2011, 2014, 2017-2019 D. R. Commander
+// Copyright (C)2009-2011, 2014, 2017-2020 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -141,6 +141,12 @@ VGLTrans::VGLTrans(void) : nprocs(fconfig.np), socket(NULL), thread(NULL),
 {
 	memset(&version, 0, sizeof(rrversion));
 	profTotal.setName("Total     ");
+	#ifdef USEHELGRIND
+	ANNOTATE_BENIGN_RACE_SIZED(&deadYet, sizeof(bool), );
+	// NOTE: Without this line, helgrind reports a data race on the class
+	// instance address in the destructor (false positive?)
+	ANNOTATE_BENIGN_RACE_SIZED(this, sizeof(VGLTrans *), );
+	#endif
 }
 
 
@@ -158,10 +164,10 @@ void VGLTrans::run(void)
 			vglout.println("[VGL] Using %d compression threads on %d CPU cores",
 				nprocs, NumProcs());
 		for(i = 0; i < nprocs; i++)
-			NEWCHECK(comp[i] = new VGLTrans::Compressor(i, this));
+			comp[i] = new VGLTrans::Compressor(i, this);
 		if(nprocs > 1) for(i = 1; i < nprocs; i++)
 		{
-			NEWCHECK(cthread[i] = new Thread(comp[i]));
+			cthread[i] = new Thread(comp[i]);
 			cthread[i]->start();
 		}
 
@@ -235,7 +241,7 @@ void VGLTrans::run(void)
 		for(i = 0; i < nprocs; i++) delete comp[i];
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
 		if(thread) thread->setError(e);
 		ready.signal();
@@ -341,7 +347,7 @@ void VGLTrans::Compressor::compressSend(Frame *f, Frame *lastf)
 			}
 			Frame *tile = f->getTile(x, y, width, height);
 			CompressedFrame *ctile = NULL;
-			if(myRank > 0) { NEWCHECK(ctile = new CompressedFrame()); }
+			if(myRank > 0) { ctile = new CompressedFrame(); }
 			else ctile = &cframe;
 			profComp.startFrame();
 			*ctile = *tile;
@@ -441,7 +447,7 @@ void VGLTrans::connect(char *displayName, unsigned short port)
 		{
 			free(serverName);  serverName = strdup("localhost");
 		}
-		NEWCHECK(socket = new Socket((bool)fconfig.ssl, true));
+		socket = new Socket((bool)fconfig.ssl, true);
 		try
 		{
 			socket->connect(serverName, port);
@@ -453,7 +459,7 @@ void VGLTrans::connect(char *displayName, unsigned short port)
 			vglout.println("[VGL]    variable points to the machine on which vglclient is running.");
 			throw;
 		}
-		NEWCHECK(thread = new Thread(this));
+		thread = new Thread(this);
 		thread->start();
 	}
 	catch(...)
