@@ -1,4 +1,4 @@
-// Copyright (C)2011, 2013-2014, 2017, 2019 D. R. Commander
+// Copyright (C)2011, 2013-2014, 2017, 2019-2020 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -17,7 +17,7 @@
 
 using namespace vglutil;
 using namespace vglcommon;
-using namespace vglserver;
+using namespace vglfaker;
 
 
 VirtualPixmap::VirtualPixmap(Display *dpy_, Visual *visual, Pixmap pm) :
@@ -25,7 +25,7 @@ VirtualPixmap::VirtualPixmap(Display *dpy_, Visual *visual, Pixmap pm) :
 {
 	CriticalSection::SafeLock l(mutex);
 	profPMBlit.setName("PMap Blit ");
-	NEWCHECK(frame = new FBXFrame(dpy_, pm, visual, true));
+	frame = new FBXFrame(dpy_, pm, visual, true);
 }
 
 
@@ -36,7 +36,7 @@ VirtualPixmap::~VirtualPixmap()
 }
 
 
-int VirtualPixmap::init(int width, int height, int depth, GLXFBConfig config_,
+int VirtualPixmap::init(int width, int height, int depth, VGLFBConfig config_,
 	const int *attribs)
 {
 	if(!config_ || width < 1 || height < 1) THROW("Invalid argument");
@@ -44,12 +44,17 @@ int VirtualPixmap::init(int width, int height, int depth, GLXFBConfig config_,
 	CriticalSection::SafeLock l(mutex);
 	if(oglDraw && oglDraw->getWidth() == width && oglDraw->getHeight() == height
 		&& oglDraw->getDepth() == depth
-		&& FBCID(oglDraw->getConfig()) == FBCID(config_))
+		&& FBCID(oglDraw->getFBConfig()) == FBCID(config_))
 		return 0;
-	NEWCHECK(oglDraw = new OGLDrawable(width, height, depth, config_, attribs));
+	#ifdef EGLBACKEND
+	if(fconfig.egl)
+		oglDraw = new OGLDrawable(dpy, width, height, config_);
+	else
+	#endif
+		oglDraw = new OGLDrawable(width, height, depth, config_, attribs);
 	if(config && FBCID(config_) != FBCID(config) && ctx)
 	{
-		_glXDestroyContext(DPY3D, ctx);  ctx = 0;
+		VGLDestroyContext(dpy, ctx);  ctx = 0;
 	}
 	config = config_;
 	return 1;
@@ -68,6 +73,8 @@ Pixmap VirtualPixmap::get3DX11Pixmap(void)
 
 void VirtualPixmap::readback(void)
 {
+	if(!checkRenderMode()) return;
+
 	fconfig_reloadenv();
 
 	CriticalSection::SafeLock l(mutex);
