@@ -1,4 +1,4 @@
-// Copyright (C)2011, 2013-2014, 2019 D. R. Commander
+// Copyright (C)2011, 2013-2014, 2019-2020 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -16,7 +16,7 @@
 #include "Frame.h"
 
 
-namespace vglserver
+namespace vglfaker
 {
 	class VirtualPixmap : public VirtualDrawable
 	{
@@ -24,7 +24,7 @@ namespace vglserver
 
 			VirtualPixmap(Display *dpy, Visual *visual, Pixmap pm);
 			~VirtualPixmap();
-			int init(int width, int height, int depth, GLXFBConfig config,
+			int init(int width, int height, int depth, VGLFBConfig config,
 				const int *attribs);
 			void readback(void);
 			Pixmap get3DX11Pixmap(void);
