@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005, 2006 Sun Microsystems, Inc.
-// Copyright (C)2009-2015, 2017-2020 D. R. Commander
+// Copyright (C)2009-2015, 2017-2021 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -22,6 +22,7 @@
 
 using namespace vglutil;
 using namespace vglcommon;
+using namespace vglfaker;
 using namespace vglserver;
 
 
@@ -69,7 +70,7 @@ VirtualWin::VirtualWin(Display *dpy_, Window win) :
 	alreadyWarnedPluginRenderMode = false;
 	XWindowAttributes xwa;
 	if(!XGetWindowAttributes(dpy, win, &xwa) || !xwa.visual)
-		throw(vglutil::Error(__FUNCTION__, "Invalid window", -1));
+		throw(Error(__FUNCTION__, "Invalid window", -1));
 	if(!fconfig.wm && !(xwa.your_event_mask & StructureNotifyMask))
 	{
 		if(!(eventdpy = _XOpenDisplay(DisplayString(dpy))))
@@ -79,7 +80,7 @@ VirtualWin::VirtualWin(Display *dpy_, Window win) :
 			vglout.println("[VGL] Selecting structure notify events in window 0x%.8x",
 				win);
 	}
-	stereoVisual = glxvisual::visAttrib2D(dpy, DefaultScreen(dpy),
+	stereoVisual = glxvisual::visAttrib(dpy, DefaultScreen(dpy),
 		xwa.visual->visualid, GLX_STEREO);
 }
 
@@ -99,10 +100,10 @@ VirtualWin::~VirtualWin(void)
 		{
 			delete plugin;  plugin = NULL;
 		}
-		catch(Error &e)
+		catch(std::exception &e)
 		{
 			if(fconfig.verbose)
-				vglout.println("[VGL] WARNING: %s", e.getMessage());
+				vglout.println("[VGL] WARNING: %s", e.what());
 		}
 	}
 	if(eventdpy) { _XCloseDisplay(eventdpy);  eventdpy = NULL; }
@@ -110,7 +111,7 @@ VirtualWin::~VirtualWin(void)
 }
 
 
-int VirtualWin::init(int w, int h, GLXFBConfig config_)
+int VirtualWin::init(int w, int h, VGLFBConfig config_)
 {
 	CriticalSection::SafeLock l(mutex);
 	if(doWMDelete) THROW("Window has been deleted by window manager");
@@ -139,7 +140,7 @@ void VirtualWin::resize(int width, int height)
 // The FB config change doesn't actually occur until the next time
 // updatedrawable() is called
 
-void VirtualWin::checkConfig(GLXFBConfig config_)
+void VirtualWin::checkConfig(VGLFBConfig config_)
 {
 	CriticalSection::SafeLock l(mutex);
 	if(doWMDelete) THROW("Window has been deleted by window manager");
@@ -166,7 +167,7 @@ void VirtualWin::cleanup(void)
 }
 
 
-void VirtualWin::initFromWindow(GLXFBConfig config_)
+void VirtualWin::initFromWindow(VGLFBConfig config_)
 {
 	XSync(dpy, False);
 	XWindowAttributes xwa;
@@ -231,7 +232,14 @@ void VirtualWin::swapBuffers(void)
 {
 	CriticalSection::SafeLock l(mutex);
 	if(doWMDelete) THROW("Window has been deleted by window manager");
-	if(oglDraw) oglDraw->swap();
+	if(oglDraw)
+	{
+		if(fconfig.amdgpuHack)
+			copyPixels(0, 0, oglDraw->getWidth(), oglDraw->getHeight(), 0, 0,
+				getGLXDrawable(), GL_BACK, GL_FRONT);
+		else
+			oglDraw->swap();
+	}
 }
 
 
@@ -254,7 +262,8 @@ void VirtualWin::readback(GLint drawBuf, bool spoilLast, bool sync)
 	fconfig_reloadenv();
 	bool doStereo = false;  int stereoMode = fconfig.stereo;
 
-	if(fconfig.readback == RRREAD_NONE) return;
+	if(fconfig.readback == RRREAD_NONE || !checkRenderMode())
+		return;
 
 	CriticalSection::SafeLock l(mutex);
 	if(doWMDelete) THROW("Window has been deleted by window manager");
@@ -322,7 +331,7 @@ void VirtualWin::readback(GLint drawBuf, bool spoilLast, bool sync)
 		case RRCOMP_YUV:
 			if(!vglconn)
 			{
-				NEWCHECK(vglconn = new VGLTrans());
+				vglconn = new VGLTrans();
 				vglconn->connect(
 					strlen(fconfig.client) > 0 ? fconfig.client : DisplayString(dpy),
 					fconfig.port);
@@ -359,17 +368,9 @@ TempContext *VirtualWin::setupPluginTempContext(GLint drawBuf)
 	}
 	else
 	{
-		if(!ctx)
-		{
-			if(!isInit())
-				THROW("VirtualDrawable instance has not been fully initialized");
-			if((ctx = _glXCreateNewContext(DPY3D, config, GLX_RGBA_TYPE, NULL,
-				direct)) == 0)
-				THROW("Could not create OpenGL context for readback");
-		}
-		tc = new TempContext(DPY3D, getGLXDrawable(), getGLXDrawable(), ctx,
-			config, GLX_RGBA_TYPE);
-		_glReadBuffer(drawBuf);
+		initReadbackContext();
+		tc = new TempContext(dpy, getGLXDrawable(), getGLXDrawable(), ctx);
+		VGLReadBuffer(drawBuf);
 	}
 
 	return tc;
@@ -389,7 +390,7 @@ void VirtualWin::sendPlugin(GLint drawBuf, bool spoilLast, bool sync,
 		if(!plugin)
 		{
 			tc = setupPluginTempContext(drawBuf);
-			NEWCHECK(plugin = new TransPlugin(dpy, x11Draw, fconfig.transport));
+			plugin = new TransPlugin(dpy, x11Draw, fconfig.transport);
 			plugin->connect(
 				strlen(fconfig.client) > 0 ? fconfig.client : DisplayString(dpy),
 				fconfig.port);
@@ -529,7 +530,7 @@ void VirtualWin::sendX11(GLint drawBuf, bool spoilLast, bool sync,
 	int width = oglDraw->getWidth(), height = oglDraw->getHeight();
 
 	FBXFrame *f;
-	if(!x11trans) NEWCHECK(x11trans = new X11Trans());
+	if(!x11trans) x11trans = new X11Trans();
 	if(spoilLast && fconfig.spoil && !x11trans->isReady()) return;
 	if(!fconfig.spoil) x11trans->synchronize();
 	ERRIFNOT(f = x11trans->getFrame(dpy, x11Draw, width, height));
@@ -567,7 +568,7 @@ void VirtualWin::sendXV(GLint drawBuf, bool spoilLast, bool sync,
 	int width = oglDraw->getWidth(), height = oglDraw->getHeight();
 
 	XVFrame *f;
-	if(!xvtrans) NEWCHECK(xvtrans = new XVTrans());
+	if(!xvtrans) xvtrans = new XVTrans();
 	if(spoilLast && fconfig.spoil && !xvtrans->isReady()) return;
 	if(!fconfig.spoil) xvtrans->synchronize();
 	ERRIFNOT(f = xvtrans->getFrame(dpy, x11Draw, width, height));
