@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005, 2006 Sun Microsystems, Inc.
-// Copyright (C)2011, 2014, 2019 D. R. Commander
+// Copyright (C)2011, 2014, 2019-2020 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -23,7 +23,7 @@
 
 // This maps a window ID to an off-screen drawable instance
 
-namespace vglserver
+namespace vglfaker
 {
 	class WindowHash : public HASH
 	{
@@ -49,40 +49,15 @@ namespace vglserver
 					free(dpystring);
 			}
 
-			VirtualWin *find(Display *dpy, Window win)
+			// dpy == NULL: search for VirtualWin instance by off-screen drawable ID
+			// dpy == non-NULL: search for VirtualWin instance by Window ID
+			VirtualWin *find(Display *dpy, GLXDrawable glxd)
 			{
-				if(!dpy || !win) return NULL;
-				return HASH::find(DisplayString(dpy), win);
+				if(!glxd) return NULL;
+				return HASH::find(dpy ? DisplayString(dpy) : NULL, glxd);
 			}
 
-			bool find(Display *dpy, GLXDrawable glxd, VirtualWin* &vwin)
-			{
-				VirtualWin *vw;
-				if(!dpy || !glxd) return false;
-				vw = HASH::find(DisplayString(dpy), glxd);
-				if(vw == NULL || vw == (VirtualWin *)-1) return false;
-				else { vwin = vw;  return true; }
-			}
-
-			bool isOverlay(Display *dpy, GLXDrawable glxd)
-			{
-				VirtualWin *vw;
-				if(!dpy || !glxd) return false;
-				vw = HASH::find(DisplayString(dpy), glxd);
-				if(vw == (VirtualWin *)-1) return true;
-				return false;
-			}
-
-			bool find(GLXDrawable glxd, VirtualWin* &vwin)
-			{
-				VirtualWin *vw;
-				if(!glxd) return false;
-				vw = HASH::find(NULL, glxd);
-				if(vw == NULL || vw == (VirtualWin *)-1) return false;
-				else { vwin = vw;  return true; }
-			}
-
-			VirtualWin *initVW(Display *dpy, Window win, GLXFBConfig config)
+			VirtualWin *initVW(Display *dpy, Window win, VGLFBConfig config)
 			{
 				if(!dpy || !win || !config) THROW("Invalid argument");
 				HashEntry *ptr = NULL;
@@ -91,7 +66,7 @@ namespace vglserver
 				{
 					if(!ptr->value)
 					{
-						NEWCHECK(ptr->value = new VirtualWin(dpy, win));
+						ptr->value = new VirtualWin(dpy, win);
 						VirtualWin *vw = ptr->value;
 						vw->initFromWindow(config);
 					}
@@ -105,17 +80,6 @@ namespace vglserver
 				return NULL;
 			}
 
-			void setOverlay(Display *dpy, Window win)
-			{
-				if(!dpy || !win) return;
-				HashEntry *ptr = NULL;
-				vglutil::CriticalSection::SafeLock l(mutex);
-				if((ptr = HASH::findEntry(DisplayString(dpy), win)) != NULL)
-				{
-					if(!ptr->value) ptr->value = (VirtualWin *)-1;
-				}
-			}
-
 			void remove(Display *dpy, GLXDrawable glxd)
 			{
 				if(!dpy || !glxd) return;
@@ -132,7 +96,7 @@ namespace vglserver
 				{
 					VirtualWin *vw = ptr->value;
 					next = ptr->next;
-					if(vw && vw != (VirtualWin *)-1 && dpy == vw->getX11Display())
+					if(vw && dpy == vw->getX11Display())
 						HASH::killEntry(ptr);
 					ptr = next;
 				}
@@ -150,7 +114,7 @@ namespace vglserver
 				if(entry)
 				{
 					free(entry->key1);
-					if(entry->value != (VirtualWin *)-1) delete entry->value;
+					delete entry->value;
 				}
 			}
 
@@ -160,14 +124,12 @@ namespace vglserver
 				return (
 					// Match 2D X Server display string and Window ID stored in
 					// VirtualDrawable instance
-					(vw && vw != (VirtualWin *)-1 && key1
-						&& !strcasecmp(DisplayString(vw->getX11Display()), key1)
+					(vw && key1 && !strcasecmp(DisplayString(vw->getX11Display()), key1)
 						&& key2 == vw->getX11Drawable())
 					||
 					// If key1 is NULL, match off-screen drawable ID instead of X Window
 					// ID
-					(vw && vw != (VirtualWin *)-1 && key1 == NULL
-						&& key2 == vw->getGLXDrawable())
+					(vw && key1 == NULL && key2 == vw->getGLXDrawable())
 					||
 					// Direct match
 					(key1 && !strcasecmp(key1, entry->key1) && key2 == entry->key2)
@@ -182,6 +144,6 @@ namespace vglserver
 #undef HASH
 
 
-#define winhash  (*(WindowHash::getInstance()))
+#define winhash  (*(vglfaker::WindowHash::getInstance()))
 
 #endif  // __WINDOWHASH_H__
