@@ -8,6 +8,7 @@
 		glXDestroyContext;
 		glXDestroyGLXPixmap;
 		glXGetConfig;
+		glXGetCurrentContext;
 		glXGetCurrentDrawable;
 		glXIsDirect;
 		glXMakeCurrent;
@@ -55,6 +56,7 @@
 
 		/* GLX_EXT_import_context */
 		glXFreeContextEXT;
+		glXGetCurrentDisplayEXT;
 		glXImportContextEXT;
 		glXQueryContextInfoEXT;
 
@@ -65,14 +67,6 @@
 		glXBindTexImageEXT;
 		glXReleaseTexImageEXT;
 
-		/* GLX_NV_swap_group */
-		glXBindSwapBarrierNV;
-		glXJoinSwapGroupNV;
-		glXQueryFrameCountNV;
-		glXQueryMaxSwapGroupsNV;
-		glXQuerySwapGroupNV;
-		glXResetFrameCountNV;
-
 		/* GLX_SGI_make_current_read */
 		glXGetCurrentReadDrawableSGI;
 		glXMakeCurrentReadSGI;
@@ -95,17 +89,36 @@
 		glXQueryGLXPbufferSGIX;
 		glXSelectEventSGIX;
 
-		/* GLX_SUN_get_transparent_index */
-		glXGetTransparentIndexSUN;
-
 		/* OpenGL */
+		glBindFramebuffer;
+		glBindFramebufferEXT;
+		glDeleteFramebuffers;
+		glDeleteFramebuffersEXT;
 		glFinish;
 		glFlush;
 		glDrawBuffer;
 		glDrawBuffers;
+		glDrawBuffersARB;
+		glDrawBuffersATI;
+		glFramebufferDrawBufferEXT;
+		glFramebufferDrawBuffersEXT;
+		glFramebufferReadBufferEXT;
+		glGetBooleanv;
+		glGetDoublev;
+		glGetFloatv;
+		glGetFramebufferAttachmentParameteriv;
+		glGetFramebufferParameteriv;
+		glGetIntegerv;
+		glGetInteger64v;
+		glGetNamedFramebufferParameteriv;
 		glGetString;
 		glGetStringi;
+		glNamedFramebufferDrawBuffer;
+		glNamedFramebufferDrawBuffers;
+		glNamedFramebufferReadBuffer;
 		glPopAttrib;
+		glReadBuffer;
+		glReadPixels;
 		glViewport;
 
 		/* OpenCL */
@@ -133,6 +146,7 @@
 		XMoveResizeWindow;
 		XNextEvent;
 		XOpenDisplay;
+		XkbOpenDisplay;
 		XQueryExtension;
 		XResizeWindow;
 		XServerVendor;
@@ -141,8 +155,6 @@
 		#endif
 		XWindowEvent;
 
-		XCopyArea_FBX;
-
 		_vgl_dlopen;
 		_vgl_getAutotestColor;
 		_vgl_getAutotestFrame;
