@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005, 2006 Sun Microsystems, Inc.
-// Copyright (C)2009, 2011, 2013-2016, 2018-2020 D. R. Commander
+// Copyright (C)2009, 2011, 2013-2016, 2018-2021 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -19,6 +19,11 @@
 #define GL_GLEXT_PROTOTYPES
 #define GLX_GLXEXT_PROTOTYPES
 #include <GL/glx.h>
+#ifdef EGLBACKEND
+#include <EGL/egl.h>
+#define EGL_EGLEXT_PROTOTYPES
+#include <EGL/eglext.h>
+#endif
 #ifdef FAKEOPENCL
 #include <CL/opencl.h>
 #endif
@@ -35,10 +40,17 @@ extern "C" {
 }
 #endif
 #include "faker.h"
+#include <X11/XKBlib.h>
+#ifdef USEHELGRIND
+	#include <valgrind/helgrind.h>
+#else
+	#define ANNOTATE_BENIGN_RACE_SIZED(a, b, c) {}
+#endif
 
 
 #define CHECKSYM_NONFATAL(s) \
 { \
+	ANNOTATE_BENIGN_RACE_SIZED(&__##s, sizeof(_##s##Type), ); \
 	if(!__##s) \
 	{ \
 		vglfaker::init(); \
@@ -49,6 +61,7 @@ extern "C" {
 
 #define CHECKSYM(s, fake_s) \
 { \
+	ANNOTATE_BENIGN_RACE_SIZED(&__##s, sizeof(_##s##Type), ); \
 	if(!__##s) \
 	{ \
 		vglfaker::init(); \
@@ -308,6 +321,20 @@ extern "C" {
 		return retval; \
 	}
 
+#define VFUNCDEF10(f, at1, a1, at2, a2, at3, a3, at4, a4, at5, a5, at6, a6, \
+	at7, a7, at8, a8, at9, a9, at10, a10, fake_f) \
+	typedef void (*_##f##Type)(at1, at2, at3, at4, at5, at6, at7, at8, at9, \
+		at10); \
+	SYMDEF(f); \
+	static INLINE void _##f(at1 a1, at2 a2, at3 a3, at4 a4, at5 a5, at6 a6, \
+		at7 a7, at8 a8, at9 a9, at10 a10) \
+	{ \
+		CHECKSYM(f, fake_f); \
+		DISABLE_FAKER(); \
+		__##f(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10); \
+		ENABLE_FAKER(); \
+	}
+
 #define FUNCDEF12(RetType, f, at1, a1, at2, a2, at3, a3, at4, a4, at5, a5, \
 	at6, a6, at7, a7, at8, a8, at9, a9, at10, a10, at11, a11, at12, a12, \
 	fake_f) \
@@ -354,6 +381,8 @@ VFUNCDEF2(glXDestroyGLXPixmap, Display *, dpy, GLXPixmap, pix,
 FUNCDEF4(int, glXGetConfig, Display *, dpy, XVisualInfo *, vis, int, attrib,
 	int *, value, glXGetConfig)
 
+FUNCDEF0(GLXContext, glXGetCurrentContext, glXGetCurrentContext)
+
 FUNCDEF0(GLXDrawable, glXGetCurrentDrawable, glXGetCurrentDrawable)
 
 FUNCDEF2(Bool, glXIsDirect, Display *, dpy, GLXContext, ctx, glXIsDirect)
@@ -501,27 +530,6 @@ VFUNCDEF3(glXReleaseTexImageEXT, Display *, dpy, GLXDrawable, drawable,
 	int, buffer, glXReleaseTexImageEXT)
 
 
-// GLX_NV_swap_group
-
-FUNCDEF3(Bool, glXJoinSwapGroupNV, Display *, dpy, GLXDrawable, drawable,
-	GLuint, group, glXJoinSwapGroupNV)
-
-FUNCDEF3(Bool, glXBindSwapBarrierNV, Display *, dpy, GLuint, group,
-	GLuint, barrier, glXBindSwapBarrierNV)
-
-FUNCDEF4(Bool, glXQuerySwapGroupNV, Display *, dpy, GLXDrawable, drawable,
-	GLuint *, group, GLuint *, barrier, glXQuerySwapGroupNV)
-
-FUNCDEF4(Bool, glXQueryMaxSwapGroupsNV, Display *, dpy, int, screen,
-	GLuint *, maxGroups, GLuint *, maxBarriers, glXQueryMaxSwapGroupsNV)
-
-FUNCDEF3(Bool, glXQueryFrameCountNV, Display *, dpy, int, screen,
-	GLuint *, count, glXQueryFrameCountNV)
-
-FUNCDEF2(Bool, glXResetFrameCountNV, Display *, dpy, int, screen,
-	glXResetFrameCountNV)
-
-
 // GLX_SGI_swap_control
 
 FUNCDEF1(int, glXSwapIntervalSGI, int, interval, glXSwapIntervalSGI)
@@ -533,20 +541,17 @@ FUNCDEF2(GLXFBConfigSGIX, glXGetFBConfigFromVisualSGIX, Display *, dpy,
 	XVisualInfo *, vis, glXGetFBConfigFromVisualSGIX)
 
 
-// GLX_SUN_get_transparent_index
-
-#if defined(GLX_GLXEXT_VERSION) && GLX_GLXEXT_VERSION >= 20190000
-FUNCDEF4(int, glXGetTransparentIndexSUN, Display *, dpy, Window, overlay,
-	Window, underlay, unsigned long *, transparentIndex,
-	glXGetTransparentIndexSUN)
-#else
-FUNCDEF4(int, glXGetTransparentIndexSUN, Display *, dpy, Window, overlay,
-	Window, underlay, long *, transparentIndex, glXGetTransparentIndexSUN)
-#endif
-
-
 // GL functions
 
+VFUNCDEF2(glBindFramebuffer, GLenum, target, GLuint, framebuffer,
+	glBindFramebuffer)
+
+VFUNCDEF2(glBindFramebufferEXT, GLenum, target, GLuint, framebuffer,
+	glBindFramebufferEXT)
+
+VFUNCDEF2(glDeleteFramebuffers, GLsizei, n, const GLuint *, framebuffers,
+	glDeleteFramebuffers)
+
 VFUNCDEF0(glFinish, glFinish)
 
 VFUNCDEF0(glFlush, glFlush)
@@ -555,13 +560,56 @@ VFUNCDEF1(glDrawBuffer, GLenum, drawbuf, glDrawBuffer)
 
 VFUNCDEF2(glDrawBuffers, GLsizei, n, const GLenum *, bufs, glDrawBuffers)
 
+VFUNCDEF2(glFramebufferDrawBufferEXT, GLuint, framebuffer, GLenum, mode,
+	glFramebufferDrawBufferEXT)
+
+VFUNCDEF3(glFramebufferDrawBuffersEXT, GLuint, framebuffer, GLsizei, n,
+	const GLenum *, bufs, glFramebufferDrawBuffersEXT)
+
+VFUNCDEF2(glFramebufferReadBufferEXT, GLuint, framebuffer, GLenum, mode,
+	glFramebufferReadBufferEXT)
+
+VFUNCDEF2(glGetBooleanv, GLenum, pname, GLboolean *, data, glGetBooleanv)
+
+VFUNCDEF2(glGetDoublev, GLenum, pname, GLdouble *, data, glGetDoublev)
+
+VFUNCDEF2(glGetFloatv, GLenum, pname, GLfloat *, data, glGetFloatv)
+
+VFUNCDEF4(glGetFramebufferAttachmentParameteriv, GLenum, target,
+	GLenum, attachment, GLenum, pname, GLint *, params,
+	glGetFramebufferAttachmentParameteriv)
+
+VFUNCDEF3(glGetFramebufferParameteriv, GLenum, target, GLenum, pname,
+	GLint *, params, glGetFramebufferParameteriv)
+
+VFUNCDEF2(glGetIntegerv, GLenum, pname, GLint *, params, glGetIntegerv)
+
+VFUNCDEF2(glGetInteger64v, GLenum, pname, GLint64 *, data, glGetInteger64v)
+
+VFUNCDEF3(glGetNamedFramebufferParameteriv, GLuint, framebuffer,
+	GLenum, pname, GLint *, param, glGetNamedFramebufferParameteriv)
+
 FUNCDEF1(const GLubyte *, glGetString, GLenum, name, glGetString)
 
 FUNCDEF2(const GLubyte *, glGetStringi, GLenum, name, GLuint, index,
 	glGetStringi)
 
+VFUNCDEF2(glNamedFramebufferDrawBuffer, GLuint, framebuffer, GLenum, buf,
+	glNamedFramebufferDrawBuffer)
+
+VFUNCDEF3(glNamedFramebufferDrawBuffers, GLuint, framebuffer, GLsizei, n,
+	const GLenum *, bufs, glNamedFramebufferDrawBuffers)
+
+VFUNCDEF2(glNamedFramebufferReadBuffer, GLuint, framebuffer, GLenum, mode,
+	glNamedFramebufferReadBuffer)
+
 VFUNCDEF0(glPopAttrib, glPopAttrib)
 
+VFUNCDEF1(glReadBuffer, GLenum, mode, glReadBuffer)
+
+VFUNCDEF7(glReadPixels, GLint, x, GLint, y, GLsizei, width, GLsizei, height,
+	GLenum, format, GLenum, type, GLvoid *, pixels, glReadPixels)
+
 VFUNCDEF4(glViewport, GLint, x, GLint, y, GLsizei, width, GLsizei, height,
 	glViewport)
 
@@ -641,6 +689,10 @@ FUNCDEF2(int, XNextEvent, Display *, dpy, XEvent *, xe, XNextEvent)
 
 FUNCDEF1(Display *, XOpenDisplay, _Xconst char *, name, XOpenDisplay)
 
+FUNCDEF6(Display *, XkbOpenDisplay, char *, display_name, int *, event_rtrn,
+	int *, error_rtrn, int *, major_in_out, int *, minor_in_out,
+	int *, reason_rtrn, XkbOpenDisplay)
+
 FUNCDEF5(Bool, XQueryExtension, Display *, dpy, _Xconst char *, name,
 	int *, major_opcode, int *, first_event, int *, first_error,
 	XQueryExtension)
@@ -695,13 +747,21 @@ FUNCDEF1(xcb_generic_event_t *, xcb_wait_for_event, xcb_connection_t *, conn,
 
 VFUNCDEF2(glBindBuffer, GLenum, target, GLuint, buffer, NULL)
 
+VFUNCDEF2(glBindRenderbuffer, GLenum, target, GLuint, renderbuffer, NULL)
+
 VFUNCDEF7(glBitmap, GLsizei, width, GLsizei, height, GLfloat, xorig,
 	GLfloat, yorig, GLfloat, xmove, GLfloat, ymove, const GLubyte *, bitmap,
 	NULL)
 
+VFUNCDEF10(glBlitFramebuffer, GLint, srcX0, GLint, srcY0, GLint, srcX1,
+	GLint, srcY1, GLint, dstX0, GLint, dstY0, GLint, dstX1, GLint, dstY1,
+	GLbitfield, mask, GLenum, filter, NULL)
+
 VFUNCDEF4(glBufferData, GLenum, target, GLsizeiptr, size, const GLvoid *, data,
 	GLenum, usage, NULL)
 
+FUNCDEF1(GLenum, glCheckFramebufferStatus, GLenum, target, NULL)
+
 VFUNCDEF1(glClear, GLbitfield, mask, NULL)
 
 VFUNCDEF4(glClearColor, GLclampf, red, GLclampf, green, GLclampf, blue,
@@ -710,19 +770,25 @@ VFUNCDEF4(glClearColor, GLclampf, red, GLclampf, green, GLclampf, blue,
 VFUNCDEF5(glCopyPixels, GLint, x, GLint, y, GLsizei, width, GLsizei, height,
 	GLenum, type, NULL)
 
+VFUNCDEF2(glDeleteRenderbuffers, GLsizei, n, const GLuint *, renderbuffers,
+	NULL)
+
 VFUNCDEF0(glEndList, NULL)
 
+VFUNCDEF4(glFramebufferRenderbuffer, GLenum, target, GLenum, attachment,
+	GLenum, renderbuffertarget, GLuint, renderbuffer, NULL)
+
 VFUNCDEF2(glGenBuffers, GLsizei, n, GLuint *, buffers, NULL)
 
+VFUNCDEF2(glGenFramebuffers, GLsizei, n, GLuint *, ids, NULL)
+
+VFUNCDEF2(glGenRenderbuffers, GLsizei, n, GLuint *, renderbuffers, NULL)
+
 VFUNCDEF3(glGetBufferParameteriv, GLenum, target, GLenum, value, GLint *, data,
 	NULL)
 
 FUNCDEF0(GLenum, glGetError, NULL)
 
-VFUNCDEF2(glGetFloatv, GLenum, pname, GLfloat *, params, NULL)
-
-VFUNCDEF2(glGetIntegerv, GLenum, pname, GLint *, params, NULL)
-
 VFUNCDEF0(glLoadIdentity, NULL)
 
 FUNCDEF2(void *, glMapBuffer, GLenum, target, GLenum, access, NULL)
@@ -742,14 +808,59 @@ VFUNCDEF0(glPushMatrix, NULL)
 
 VFUNCDEF2(glRasterPos2i, GLint, x, GLint, y, NULL)
 
-VFUNCDEF1(glReadBuffer, GLenum, mode, NULL)
+VFUNCDEF4(glRenderbufferStorage, GLenum, target, GLenum, internalformat,
+	GLsizei, width, GLsizei, height, NULL)
 
-VFUNCDEF7(glReadPixels, GLint, x, GLint, y, GLsizei, width, GLsizei, height,
-	GLenum, format, GLenum, type, GLvoid *, pixels, NULL)
+VFUNCDEF5(glRenderbufferStorageMultisample, GLenum, target, GLsizei, samples,
+	GLenum, internalformat, GLsizei, width, GLsizei, height, NULL)
 
 FUNCDEF1(GLboolean, glUnmapBuffer, GLenum, target, NULL)
 
-FUNCDEF0(GLXContext, glXGetCurrentContext, NULL)
+// EGL functions used by the faker (but not interposed.)
+
+#ifdef EGLBACKEND
+
+FUNCDEF1(EGLBoolean, eglBindAPI, EGLenum, api, NULL)
+
+FUNCDEF4(EGLContext, eglCreateContext, EGLDisplay, display, EGLConfig, config,
+	EGLContext, share_context, EGLint const *, attrib_list, NULL)
+
+FUNCDEF2(EGLBoolean, eglDestroyContext, EGLDisplay, display,
+	EGLContext, context, NULL)
+
+FUNCDEF0(EGLContext, eglGetCurrentContext, NULL)
+
+FUNCDEF0(EGLint, eglGetError, NULL)
+
+FUNCDEF3(EGLDisplay, eglGetPlatformDisplayEXT, EGLenum, platform, void *,
+	native_display, const EGLint *, attrib_list, NULL)
+
+typedef void (*(*_eglGetProcAddressType)(const char *))(void);
+SYMDEF(eglGetProcAddress);
+static INLINE void (*_eglGetProcAddress(const char *procName))(void)
+{
+	CHECKSYM(eglGetProcAddress, NULL);
+	return __eglGetProcAddress(procName);
+}
+
+FUNCDEF3(EGLBoolean, eglInitialize, EGLDisplay, display, EGLint *, major,
+	EGLint *, minor, NULL)
+
+FUNCDEF4(EGLBoolean, eglMakeCurrent, EGLDisplay, display, EGLSurface, draw,
+	EGLSurface, read, EGLContext, context, NULL)
+
+FUNCDEF3(EGLBoolean, eglQueryDevicesEXT, EGLint, max_devices, EGLDeviceEXT *,
+	devices, EGLint *, num_devices, NULL)
+
+FUNCDEF2(const char *, eglQueryDeviceStringEXT, EGLDeviceEXT, device, EGLint,
+	name, NULL)
+
+FUNCDEF2(char const *, eglQueryString, EGLDisplay, display, EGLint, name,
+	NULL)
+
+FUNCDEF1(EGLBoolean, eglTerminate, EGLDisplay, display, NULL)
+
+#endif
 
 // We load all XCB functions dynamically, so that the same VirtualGL binary
 // can be used to support systems with and without XCB libraries.
@@ -790,19 +901,4 @@ FUNCDEF3(xcb_keysym_t, xcb_key_symbols_get_keysym, xcb_key_symbols_t *, syms,
 #endif
 
 
-static INLINE int DrawingToFront(void)
-{
-	GLint drawbuf = GL_BACK;
-	_glGetIntegerv(GL_DRAW_BUFFER, &drawbuf);
-	return IS_FRONT(drawbuf);
-}
-
-
-static INLINE int DrawingToRight(void)
-{
-	GLint drawbuf = GL_LEFT;
-	_glGetIntegerv(GL_DRAW_BUFFER, &drawbuf);
-	return IS_RIGHT(drawbuf);
-}
-
 #endif  // __FAKER_SYM_H__
