@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005, 2006 Sun Microsystems, Inc.
-// Copyright (C)2009, 2011-2016, 2018-2019 D. R. Commander
+// Copyright (C)2009, 2011-2016, 2018-2021 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -22,8 +22,6 @@
 #endif
 #include "keycodetokeysym.h"
 
-using namespace vglserver;
-
 
 // Interposed X11 functions
 
@@ -49,6 +47,8 @@ int XCloseDisplay(Display *dpy)
 
 		OPENTRACE(XCloseDisplay);  PRARGD(dpy);  STARTTRACE();
 
+	DISABLE_FAKER();
+
 	#ifdef FAKEXCB
 	if(fconfig.fakeXCB)
 	{
@@ -68,12 +68,12 @@ int XCloseDisplay(Display *dpy)
 	#endif
 
 	winhash.remove(dpy);
-	dpyhash.remove(dpy);
 	retval = _XCloseDisplay(dpy);
 
 		STOPTRACE();  CLOSETRACE();
 
 	CATCH();
+	ENABLE_FAKER();
 	return retval;
 }
 
@@ -89,7 +89,9 @@ int XCopyArea(Display *dpy, Drawable src, Drawable dst, GC gc, int src_x,
 		return _XCopyArea(dpy, src, dst, gc, src_x, src_y, width, height, dest_x,
 			dest_y);
 
-	VirtualDrawable *srcVW = NULL;  VirtualDrawable *dstVW = NULL;
+	DISABLE_FAKER();
+
+	vglfaker::VirtualDrawable *srcVW = NULL, *dstVW = NULL;
 	bool srcWin = false, dstWin = false;
 	bool copy2d = true, copy3d = false, triggerRB = false;
 	GLXDrawable glxsrc = 0, glxdst = 0;
@@ -100,9 +102,9 @@ int XCopyArea(Display *dpy, Drawable src, Drawable dst, GC gc, int src_x,
 		PRARGX(gc);  PRARGI(src_x);  PRARGI(src_y);  PRARGI(width);
 		PRARGI(height);  PRARGI(dest_x);  PRARGI(dest_y);  STARTTRACE();
 
-	if(!(srcVW = (VirtualDrawable *)pmhash.find(dpy, src)))
+	if(!(srcVW = (vglfaker::VirtualDrawable *)pmhash.find(dpy, src)))
 	{
-		srcVW = (VirtualDrawable *)winhash.find(dpy, src);
+		srcVW = (vglfaker::VirtualDrawable *)winhash.find(dpy, src);
 		if(srcVW) srcWin = true;
 	}
 	if(srcVW && !srcVW->isInit())
@@ -112,9 +114,9 @@ int XCopyArea(Display *dpy, Drawable src, Drawable dst, GC gc, int src_x,
 		srcVW = NULL;
 		srcWin = false;
 	}
-	if(!(dstVW = (VirtualDrawable *)pmhash.find(dpy, dst)))
+	if(!(dstVW = (vglfaker::VirtualDrawable *)pmhash.find(dpy, dst)))
 	{
-		dstVW = (VirtualDrawable *)winhash.find(dpy, dst);
+		dstVW = (vglfaker::VirtualDrawable *)winhash.find(dpy, dst);
 		if(dstVW) dstWin = true;
 	}
 	if(dstVW && !dstVW->isInit())
@@ -126,7 +128,8 @@ int XCopyArea(Display *dpy, Drawable src, Drawable dst, GC gc, int src_x,
 	// GLX (3D) Pixmap --> non-GLX (2D) drawable
 	// Sync pixels from the 3D pixmap (on the 3D X Server) to the corresponding
 	// 2D pixmap (on the 2D X Server) and let the "real" XCopyArea() do the rest.
-	if(srcVW && !srcWin && !dstVW) ((VirtualPixmap *)srcVW)->readback();
+	if(srcVW && !srcWin && !dstVW)
+		((vglfaker::VirtualPixmap *)srcVW)->readback();
 
 	// non-GLX (2D) drawable --> non-GLX (2D) drawable
 	// Source and destination are not backed by a drawable on the 3D X Server, so
@@ -169,26 +172,17 @@ int XCopyArea(Display *dpy, Drawable src, Drawable dst, GC gc, int src_x,
 		glxdst = dstVW->getGLXDrawable();
 		srcVW->copyPixels(src_x, src_y, width, height, dest_x, dest_y, glxdst);
 		if(triggerRB)
-			((VirtualWin *)dstVW)->readback(GL_FRONT, false, fconfig.sync);
+			((vglfaker::VirtualWin *)dstVW)->readback(GL_FRONT, false, fconfig.sync);
 	}
 
 		STOPTRACE();  if(copy3d) PRARGX(glxsrc);  if(copy3d) PRARGX(glxdst);
 		CLOSETRACE();
 
 	CATCH();
+	ENABLE_FAKER();
 	return 0;
 }
 
-// This is provided for the benefit of the FBX library, which needs to call the
-// "real" XCopyArea() function from within the faker in order to implement the
-// X11 Transport.
-int XCopyArea_FBX(Display *dpy, Drawable src, Drawable dst, GC gc, int src_x,
-	int src_y, unsigned int width, unsigned int height, int dest_x, int dest_y)
-{
-	return _XCopyArea(dpy, src, dst, gc, src_x, src_y, width, height, dest_x,
-		dest_y);
-}
-
 
 // When a window is created, add it to the hash.  A VirtualWin instance does
 // not get created and hashed to the window until/unless the window is made
@@ -259,7 +253,7 @@ static void DeleteWindow(Display *dpy, Window win, bool subOnly = false)
 		&& children && n > 0)
 	{
 		for(unsigned int i = 0; i < n; i++) DeleteWindow(dpy, children[i]);
-		XFree(children);
+		_XFree(children);
 	}
 }
 
@@ -274,12 +268,15 @@ int XDestroySubwindows(Display *dpy, Window win)
 
 		OPENTRACE(XDestroySubwindows);  PRARGD(dpy);  PRARGX(win);  STARTTRACE();
 
+	DISABLE_FAKER();
+
 	if(dpy && win) DeleteWindow(dpy, win, true);
 	retval = _XDestroySubwindows(dpy, win);
 
 		STOPTRACE();  CLOSETRACE();
 
 	CATCH();
+	ENABLE_FAKER();
 	return retval;
 }
 
@@ -294,12 +291,15 @@ int XDestroyWindow(Display *dpy, Window win)
 
 		OPENTRACE(XDestroyWindow);  PRARGD(dpy);  PRARGX(win);  STARTTRACE();
 
+	DISABLE_FAKER();
+
 	if(dpy && win) DeleteWindow(dpy, win);
 	retval = _XDestroyWindow(dpy, win);
 
 		STOPTRACE();  CLOSETRACE();
 
 	CATCH();
+	ENABLE_FAKER();
 	return retval;
 }
 
@@ -338,8 +338,8 @@ Status XGetGeometry(Display *dpy, Drawable drawable, Window *root, int *x,
 
 		OPENTRACE(XGetGeometry);  PRARGD(dpy);  PRARGX(drawable);  STARTTRACE();
 
-	VirtualWin *vw = NULL;
-	if(winhash.find(drawable, vw))
+	vglfaker::VirtualWin *vw;
+	if((vw = winhash.find(NULL, drawable)) != NULL)
 	{
 		// Apparently drawable is a GLX drawable ID that backs a window, so we need
 		// to request the geometry of the window, not the GLX drawable.  This
@@ -349,7 +349,7 @@ Status XGetGeometry(Display *dpy, Drawable drawable, Window *root, int *x,
 	}
 	ret = _XGetGeometry(dpy, drawable, root, x, y, &width, &height, border_width,
 		depth);
-	if(winhash.find(dpy, drawable, vw) && width > 0 && height > 0)
+	if((vw = winhash.find(dpy, drawable)) != NULL && width > 0 && height > 0)
 		vw->resize(width, height);
 
 		STOPTRACE();  if(root) PRARGX(*root);  if(x) PRARGI(*x);  if(y) PRARGI(*y);
@@ -382,7 +382,9 @@ XImage *XGetImage(Display *dpy, Drawable drawable, int x, int y,
 		PRARGI(y);  PRARGI(width);  PRARGI(height);  PRARGX(plane_mask);
 		PRARGI(format);  STARTTRACE();
 
-	VirtualPixmap *vpm = pmhash.find(dpy, drawable);
+	DISABLE_FAKER();
+
+	vglfaker::VirtualPixmap *vpm = pmhash.find(dpy, drawable);
 	if(vpm) vpm->readback();
 
 	xi = _XGetImage(dpy, drawable, x, y, width, height, plane_mask, format);
@@ -390,6 +392,7 @@ XImage *XGetImage(Display *dpy, Drawable drawable, int x, int y,
 		STOPTRACE();  CLOSETRACE();
 
 	CATCH();
+	ENABLE_FAKER();
 	return xi;
 }
 
@@ -456,6 +459,62 @@ char **XListExtensions(Display *dpy, int *next)
 }
 
 
+static void setupXDisplay(Display *dpy)
+{
+	XExtCodes *codes;
+	XEDataObject obj = { dpy };
+	XExtData *extData;
+	bool excludeDisplay = vglfaker::isDisplayStringExcluded(DisplayString(dpy));
+
+	// Extension code 1 stores the excluded status for a Display.
+	if(!(codes = XAddExtension(dpy))
+		|| !(extData = (XExtData *)calloc(1, sizeof(XExtData)))
+		|| !(extData->private_data = (XPointer)malloc(sizeof(bool))))
+		THROW("Memory allocation error");
+	*(bool *)extData->private_data = excludeDisplay;
+	extData->number = codes->extension;
+	XAddToExtensionList(XEHeadOfExtensionList(obj), extData);
+
+	// Extension code 2 stores the mutex for a Display.
+	if(!(codes = XAddExtension(dpy))
+		|| !(extData = (XExtData *)calloc(1, sizeof(XExtData))))
+		THROW("Memory allocation error");
+	extData->private_data = (XPointer)(new vglutil::CriticalSection());
+	extData->number = codes->extension;
+	extData->free_private = vglfaker::deleteCS;
+	XAddToExtensionList(XEHeadOfExtensionList(obj), extData);
+
+	// Extension code 3 stores the visual attribute table for a Screen.
+	if(!(codes = XAddExtension(dpy)))
+		THROW("Memory allocation error");
+
+	// Extension code 4 stores the FB config attribute table for a Screen.
+	if(!(codes = XAddExtension(dpy)))
+		THROW("Memory allocation error");
+
+	#ifdef EGLBACKEND
+	// Extension code 5 stores the RBO context instance for a Display
+	if(!(codes = XAddExtension(dpy))
+		|| !(extData = (XExtData *)calloc(1, sizeof(XExtData))))
+		THROW("Memory allocation error");
+	extData->private_data = (XPointer)(new vglfaker::EGLRBOContext());
+	extData->number = 5;
+	extData->free_private = vglfaker::deleteRBOContext;
+	XAddToExtensionList(XEHeadOfExtensionList(obj), extData);
+	#endif
+
+	if(!excludeDisplay && strlen(fconfig.vendor) > 0)
+	{
+		// Danger, Will Robinson!  We do this to prevent a small memory leak, but
+		// we can only can get away with it because we know that Xlib dynamically
+		// allocates the vendor string.  Xlib has done so for as long as VirtualGL
+		// has been around, so it seems like a safe assumption.
+		_XFree(ServerVendor(dpy));
+		ServerVendor(dpy) = strdup(fconfig.vendor);
+	}
+}
+
+
 // This is normally where VirtualGL initializes, unless a GLX function is
 // called first.
 
@@ -473,20 +532,7 @@ Display *XOpenDisplay(_Xconst char *name)
 		OPENTRACE(XOpenDisplay);  PRARGS(name);  STARTTRACE();
 
 	dpy = _XOpenDisplay(name);
-	if(dpy)
-	{
-		if(vglfaker::excludeDisplay(DisplayString(dpy)))
-			dpyhash.add(dpy);
-		else if(strlen(fconfig.vendor) > 0)
-		{
-			// Danger, Will Robinson!  We do this to prevent a small memory leak, but
-			// we can only can get away with it because we know that Xlib dynamically
-			// allocates the vendor string.  Xlib has done so for as long as
-			// VirtualGL has been around, so it seems like a safe assumption.
-			XFree(ServerVendor(dpy));
-			ServerVendor(dpy) = strdup(fconfig.vendor);
-		}
-	}
+	if(dpy) setupXDisplay(dpy);
 
 		STOPTRACE();  PRARGD(dpy);  CLOSETRACE();
 
@@ -495,6 +541,40 @@ Display *XOpenDisplay(_Xconst char *name)
 }
 
 
+// XkbOpenDisplay() calls XOpenDisplay(), but since that function call occurs
+// within libX11, VirtualGL cannot intercept it on some platforms.  Thus we
+// need to interpose XkbOpenDisplay().
+
+Display *XkbOpenDisplay(char *display_name, int *event_rtrn, int *error_rtrn,
+	int *major_in_out, int *minor_in_out, int *reason_rtrn)
+{
+	Display *dpy = NULL;
+
+	TRY();
+
+	if(vglfaker::deadYet || vglfaker::getFakerLevel() > 0)
+		return _XkbOpenDisplay(display_name, event_rtrn, error_rtrn, major_in_out,
+			minor_in_out, reason_rtrn);
+
+	vglfaker::init();
+
+		OPENTRACE(XkbOpenDisplay);  PRARGS(display_name);  STARTTRACE();
+
+	dpy = _XkbOpenDisplay(display_name, event_rtrn, error_rtrn, major_in_out,
+		minor_in_out, reason_rtrn);
+	if(dpy) setupXDisplay(dpy);
+
+		STOPTRACE();  PRARGD(dpy);  if(event_rtrn) PRARGI(*event_rtrn);
+		if(error_rtrn) PRARGI(*error_rtrn);
+		if(major_in_out) PRARGI(*major_in_out);
+		if(minor_in_out) PRARGI(*minor_in_out);
+		if(reason_rtrn) PRARGI(*reason_rtrn);  CLOSETRACE();
+
+	CATCH();
+	return dpy;
+}
+
+
 // Tell the application that the GLX extension is present, even if it isn't.
 
 Bool XQueryExtension(Display *dpy, _Xconst char *name, int *major_opcode,
@@ -502,18 +582,25 @@ Bool XQueryExtension(Display *dpy, _Xconst char *name, int *major_opcode,
 {
 	Bool retval = True;
 
+	TRY();
+
 	if(IS_EXCLUDED(dpy))
 		return _XQueryExtension(dpy, name, major_opcode, first_event, first_error);
 
 		OPENTRACE(XQueryExtension);  PRARGD(dpy);  PRARGS(name);  STARTTRACE();
 
-	if(!strcmp(name, "GLX")) dpy = DPY3D;
-	retval = _XQueryExtension(dpy, name, major_opcode, first_event, first_error);
+	if(!strcmp(name, "GLX"))
+		retval = VGLQueryExtension(dpy, major_opcode, first_event, first_error);
+	else
+		retval = _XQueryExtension(dpy, name, major_opcode, first_event,
+			first_error);
 
 		STOPTRACE();  if(major_opcode) PRARGI(*major_opcode);
 		if(first_event) PRARGI(*first_event);
 		if(first_error) PRARGI(*first_error);  CLOSETRACE();
 
+	CATCH();
+
 	return retval;
 }
 
@@ -542,14 +629,14 @@ char *XServerVendor(Display *dpy)
 
 static void handleEvent(Display *dpy, XEvent *xe)
 {
-	VirtualWin *vw = NULL;
+	vglfaker::VirtualWin *vw;
 
 	if(IS_EXCLUDED(dpy))
 		return;
 
 	if(xe && xe->type == ConfigureNotify)
 	{
-		if(winhash.find(dpy, xe->xconfigure.window, vw))
+		if((vw = winhash.find(dpy, xe->xconfigure.window)) != NULL)
 		{
 				OPENTRACE(handleEvent);  PRARGI(xe->xconfigure.width);
 				PRARGI(xe->xconfigure.height);  PRARGX(xe->xconfigure.window);
@@ -581,7 +668,7 @@ static void handleEvent(Display *dpy, XEvent *xe)
 		Atom deleteAtom = XInternAtom(dpy, "WM_DELETE_WINDOW", True);
 		if(protoAtom && deleteAtom && cme->message_type == protoAtom
 			&& cme->data.l[0] == (long)deleteAtom
-			&& winhash.find(dpy, cme->window, vw))
+			&& (vw = winhash.find(dpy, cme->window)) != NULL)
 			vw->wmDelete();
 	}
 }
@@ -654,8 +741,8 @@ int XConfigureWindow(Display *dpy, Window win, unsigned int value_mask,
 		if(values && (value_mask & CWHeight)) { PRARGI(values->height); }
 		STARTTRACE();
 
-	VirtualWin *vw = NULL;
-	if(winhash.find(dpy, win, vw) && values)
+	vglfaker::VirtualWin *vw;
+	if((vw = winhash.find(dpy, win)) != NULL && values)
 		vw->resize(value_mask & CWWidth ? values->width : 0,
 			value_mask & CWHeight ? values->height : 0);
 	retval = _XConfigureWindow(dpy, win, value_mask, values);
@@ -692,8 +779,9 @@ int XMoveResizeWindow(Display *dpy, Window win, int x, int y,
 		OPENTRACE(XMoveResizeWindow);  PRARGD(dpy);  PRARGX(win);  PRARGI(x);
 		PRARGI(y);  PRARGI(width);  PRARGI(height);  STARTTRACE();
 
-	VirtualWin *vw = NULL;
-	if(winhash.find(dpy, win, vw)) vw->resize(width, height);
+	vglfaker::VirtualWin *vw;
+	if((vw = winhash.find(dpy, win)) != NULL)
+		vw->resize(width, height);
 	retval = _XMoveResizeWindow(dpy, win, x, y, width, height);
 
 		STOPTRACE();  CLOSETRACE();
@@ -728,8 +816,9 @@ int XResizeWindow(Display *dpy, Window win, unsigned int width,
 		OPENTRACE(XResizeWindow);  PRARGD(dpy);  PRARGX(win);  PRARGI(width);
 		PRARGI(height);  STARTTRACE();
 
-	VirtualWin *vw = NULL;
-	if(winhash.find(dpy, win, vw)) vw->resize(width, height);
+	vglfaker::VirtualWin *vw;
+	if((vw = winhash.find(dpy, win)) != NULL)
+		vw->resize(width, height);
 	retval = _XResizeWindow(dpy, win, width, height);
 
 		STOPTRACE();  CLOSETRACE();
