@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005, 2006 Sun Microsystems, Inc.
-// Copyright (C)2009, 2011, 2013-2016, 2019-2020 D. R. Commander
+// Copyright (C)2009, 2011, 2013-2016, 2019-2021 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -14,12 +14,14 @@
 
 #include <unistd.h>
 #include "Mutex.h"
-#include "ConfigHash.h"
 #include "ContextHash.h"
+#ifdef EGLBACKEND
+#include "EGLContextHash.h"
+#include "VGLPbufferHash.h"
+#endif
 #include "GLXDrawableHash.h"
 #include "GlobalCriticalSection.h"
 #include "PixmapHash.h"
-#include "ReverseConfigHash.h"
 #include "VisualHash.h"
 #include "WindowHash.h"
 #include "fakerconfig.h"
@@ -29,15 +31,14 @@
 #include "faker.h"
 
 
-using namespace vglutil;
-using namespace vglserver;
-
-
 namespace vglfaker {
 
 Display *dpy3D = NULL;
 bool deadYet = false;
 char *glExtensions = NULL;
+#ifdef EGLBACKEND
+EGLint eglMajor = 0, eglMinor = 0;
+#endif
 VGL_THREAD_LOCAL(TraceLevel, long, 0)
 VGL_THREAD_LOCAL(FakerLevel, long, 0)
 VGL_THREAD_LOCAL(ExcludeCurrent, bool, false)
@@ -52,12 +53,13 @@ static void cleanup(void)
 {
 	if(PixmapHash::isAlloc()) pmhash.kill();
 	if(VisualHash::isAlloc()) vishash.kill();
-	if(ConfigHash::isAlloc()) cfghash.kill();
-	if(ReverseConfigHash::isAlloc()) rcfghash.kill();
 	if(ContextHash::isAlloc()) ctxhash.kill();
 	if(GLXDrawableHash::isAlloc()) glxdhash.kill();
 	if(WindowHash::isAlloc()) winhash.kill();
-	if(DisplayHash::isAlloc()) dpyhash.kill();
+	#ifdef EGLBACKEND
+	if(EGLContextHash::isAlloc()) ectxhash.kill();
+	if(VGLPbufferHash::isAlloc()) vpbhash.kill();
+	#endif
 	free(glExtensions);
 	unloadSymbols();
 }
@@ -90,7 +92,7 @@ class GlobalCleanup
 			vglfaker::GlobalCriticalSection *gcs =
 				vglfaker::GlobalCriticalSection::getInstance(false);
 			if(gcs) gcs->lock(false);
-			fconfig_deleteinstance();
+			fconfig_deleteinstance(gcs);
 			deadYet = true;
 			if(gcs) gcs->unlock(false);
 		}
@@ -112,25 +114,27 @@ int xhandler(Display *dpy, XErrorEvent *xe)
 }
 
 
-void sendGLXError(unsigned short minorCode, unsigned char errorCode,
+void sendGLXError(Display *dpy, CARD16 minorCode, CARD8 errorCode,
 	bool x11Error)
 {
 	xError error;
 	int majorCode, errorBase, dummy;
 
-	ERRIFNOT(_XQueryExtension(DPY3D, "GLX", &majorCode, &dummy, &errorBase));
+	ERRIFNOT(VGLQueryExtension(dpy, &majorCode, &dummy, &errorBase));
 
-	LockDisplay(dpy3D);
+	if(!fconfig.egl) dpy = DPY3D;
+
+	LockDisplay(dpy);
 
 	error.type = X_Error;
 	error.errorCode = x11Error ? errorCode : errorBase + errorCode;
-	error.sequenceNumber = dpy3D->request;
+	error.sequenceNumber = dpy->request;
 	error.resourceID = 0;
 	error.minorCode = minorCode;
 	error.majorCode = majorCode;
-	_XError(dpy3D, &error);
+	_XError(dpy, &error);
 
-	UnlockDisplay(dpy3D);
+	UnlockDisplay(dpy);
 }
 
 
@@ -170,16 +174,84 @@ Display *init3D(void)
 		GlobalCriticalSection::SafeLock l(globalMutex);
 		if(!dpy3D)
 		{
-			if(fconfig.verbose)
-				vglout.println("[VGL] Opening connection to 3D X server %s",
-					strlen(fconfig.localdpystring) > 0 ?
-					fconfig.localdpystring : "(default)");
-			if((dpy3D = _XOpenDisplay(fconfig.localdpystring)) == NULL)
+			#ifdef EGLBACKEND
+			if(fconfig.egl)
 			{
-				vglout.print("[VGL] ERROR: Could not open display %s.\n",
-					fconfig.localdpystring);
-				safeExit(1);
-				return NULL;
+				int numDevices = 0, i;
+				const char *exts = NULL;
+				EGLDeviceEXT *devices = NULL;
+
+				if(fconfig.verbose)
+					vglout.println("[VGL] Opening EGL device %s",
+						strlen(fconfig.localdpystring) > 0 ?
+						fconfig.localdpystring : "(default)");
+
+				try
+				{
+					if(!(exts = _eglQueryString(EGL_NO_DISPLAY, EGL_EXTENSIONS)))
+						THROW("Could not query EGL extensions");
+					if(!strstr(exts, "EGL_EXT_platform_device"))
+						THROW("EGL_EXT_platform_device extension not available");
+
+					if(!_eglQueryDevicesEXT(0, NULL, &numDevices) || numDevices < 1)
+						THROW("No EGL devices found");
+					if((devices =
+						(EGLDeviceEXT *)malloc(sizeof(EGLDeviceEXT) * numDevices)) == NULL)
+						THROW("Memory allocation failure");
+					if(!_eglQueryDevicesEXT(numDevices, devices, &numDevices)
+						|| numDevices < 1)
+						THROW("Could not query EGL devices");
+					for(i = 0; i < numDevices; i++)
+					{
+						EGLDisplay edpy =
+							_eglGetPlatformDisplayEXT(EGL_PLATFORM_DEVICE_EXT, devices[i],
+								NULL);
+						if(!edpy || !_eglInitialize(edpy, &eglMajor, &eglMinor))
+							continue;
+						_eglTerminate(edpy);
+						if(!strcasecmp(fconfig.localdpystring, "egl"))
+							break;
+						const char *devStr =
+							_eglQueryDeviceStringEXT(devices[i], EGL_DRM_DEVICE_FILE_EXT);
+						if(devStr && !strcmp(devStr, fconfig.localdpystring))
+							break;
+					}
+					if(i == numDevices) THROW("Invalid EGL device");
+
+					if(!(dpy3D =
+						(Display *)_eglGetPlatformDisplayEXT(EGL_PLATFORM_DEVICE_EXT,
+							devices[i], NULL)))
+						THROW("Could not open EGL display");
+					free(devices);  devices = NULL;
+					if(!_eglInitialize((EGLDisplay)dpy3D, &eglMajor, &eglMinor))
+						THROW("Could not initialize EGL");
+					if(!(exts = _eglQueryString((EGLDisplay)dpy3D, EGL_EXTENSIONS)))
+						THROW("Could not query EGL extensions");
+					if(!strstr(exts, "EGL_KHR_no_config_context"))
+						THROW("EGL_KHR_no_config_context extension not available");
+					if(!strstr(exts, "EGL_KHR_surfaceless_context"))
+						THROW("EGL_KHR_surfaceless_context extension not available");
+				}
+				catch(...)
+				{
+					if(devices) free(devices);
+					throw;
+				}
+			}
+			else  // fconfig.egl
+			#endif
+			{
+				if(fconfig.verbose)
+					vglout.println("[VGL] Opening connection to 3D X server %s",
+						strlen(fconfig.localdpystring) > 0 ?
+						fconfig.localdpystring : "(default)");
+				if((dpy3D = _XOpenDisplay(fconfig.localdpystring)) == NULL)
+				{
+					vglout.print("[VGL] ERROR: Could not open display %s.\n",
+						fconfig.localdpystring);
+					safeExit(1);
+					return NULL;
+				}
 			}
 		}
 	}
@@ -188,7 +260,7 @@ Display *init3D(void)
 }
 
 
-bool excludeDisplay(char *name)
+bool isDisplayStringExcluded(char *name)
 {
 	fconfig_reloadenv();
 
@@ -206,6 +278,27 @@ bool excludeDisplay(char *name)
 	return false;
 }
 
+
+extern "C" {
+
+int deleteCS(XExtData *extData)
+{
+	if(extData) delete (vglutil::CriticalSection *)extData->private_data;
+	return 0;
+}
+
+#ifdef EGLBACKEND
+
+int deleteRBOContext(XExtData *extData)
+{
+	if(extData) delete (EGLRBOContext *)extData->private_data;
+	return 0;
+}
+
+#endif
+
+}
+
 }  // namespace
 
 
