@@ -1,4 +1,4 @@
-// Copyright (C)2009-2019 D. R. Commander
+// Copyright (C)2009-2021 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -31,6 +31,9 @@
 	#include <X11/extensions/Xv.h>
 	#include <X11/extensions/Xvlib.h>
 #endif
+#ifdef USEHELGRIND
+	#include <valgrind/helgrind.h>
+#endif
 
 using namespace vglutil;
 
@@ -121,11 +124,11 @@ FakerConfig *fconfig_getinstance(void)
 }
 
 
-void fconfig_deleteinstance(void)
+void fconfig_deleteinstance(CriticalSection *mutex)
 {
 	if(fconfig_instance != NULL)
 	{
-		CriticalSection::SafeLock l(fcmutex, false);
+		CriticalSection::SafeLock l(mutex ? *mutex : fcmutex, false);
 		if(fconfig_instance != NULL)
 		{
 			#if FCONFIG_USESHM == 1
@@ -188,6 +191,10 @@ static void fconfig_init(void)
 	fconfig.tilesize = RR_DEFAULTTILESIZE;
 	fconfig.transpixel = -1;
 	fconfig_reloadenv();
+	#ifdef USEHELGRIND
+	ANNOTATE_BENIGN_RACE_SIZED(&fconfig.egl, sizeof(bool), );
+	ANNOTATE_BENIGN_RACE_SIZED(&fconfig.flushdelay, sizeof(double), );
+	#endif
 }
 
 
@@ -271,6 +278,7 @@ void fconfig_reloadenv(void)
 	CriticalSection::SafeLock l(fcmutex);
 
 	FETCHENV_BOOL("VGL_ALLOWINDIRECT", allowindirect);
+	FETCHENV_BOOL("VGL_AMDGPUHACK", amdgpuHack);
 	FETCHENV_BOOL("VGL_AUTOTEST", autotest);
 	FETCHENV_STR("VGL_CLIENT", client);
 	if((env = getenv("VGL_SUBSAMP")) != NULL && strlen(env) > 0)
@@ -324,22 +332,13 @@ void fconfig_reloadenv(void)
 			strncpy(fconfig.localdpystring, env, MAXSTR - 1);
 			strncpy(fconfig_env.localdpystring, env, MAXSTR - 1);
 		}
+		if((env[0] == '/' || !strnicmp(env, "EGL", 3)))
+			fconfig.egl = true;
 	}
 	FETCHENV_BOOL("VGL_DLSYM", dlsymloader);
-	if((env = getenv("VGL_DRAWABLE")) != NULL && strlen(env) > 0)
-	{
-		int drawable = -1;
-		if(!strnicmp(env, "PB", 2)) drawable = RRDRAWABLE_PBUFFER;
-		else if(!strnicmp(env, "PI", 2)) drawable = RRDRAWABLE_PIXMAP;
-		else
-		{
-			char *t = NULL;  int itemp = strtol(env, &t, 10);
-			if(t && t != env && itemp >= 0 && itemp < RR_DRAWABLEOPT)
-				drawable = itemp;
-		}
-		if(drawable >= 0 && (!fconfig_envset || fconfig_env.drawable != drawable))
-			fconfig.drawable = fconfig_env.drawable = drawable;
-	}
+	#ifdef EGLBACKEND
+	FETCHENV_STR("VGL_EGLLIB", egllib);
+	#endif
 	FETCHENV_STR("VGL_EXCLUDE", excludeddpys);
 	#ifdef FAKEXCB
 	FETCHENV_BOOL("VGL_FAKEXCB", fakeXCB);
@@ -510,7 +509,11 @@ void fconfig_setdefaultsfromdpy(Display *dpy)
 
 	if(fconfig.port < 0)
 	{
+		#ifdef USESSL
 		fconfig.port = fconfig.ssl ? RR_DEFAULTSSLPORT : RR_DEFAULTPORT;
+		#else
+		fconfig.port = RR_DEFAULTPORT;
+		#endif
 		Atom atom = None;  unsigned long n = 0, bytesLeft = 0;
 		int actualFormat = 0;  Atom actualType = None;
 		unsigned char *prop = NULL;
@@ -595,12 +598,16 @@ void fconfig_setcompress(FakerConfig &fc, int i)
 void fconfig_print(FakerConfig &fc)
 {
 	PRCONF_INT(allowindirect);
+	PRCONF_INT(amdgpuHack);
 	PRCONF_STR(client);
 	PRCONF_INT(compress);
 	PRCONF_STR(config);
 	PRCONF_STR(defaultfbconfig);
 	PRCONF_INT(dlsymloader);
-	PRCONF_INT(drawable);
+	#ifdef EGLBACKEND
+	PRCONF_INT(egl);
+	PRCONF_STR(egllib);
+	#endif
 	PRCONF_STR(excludeddpys);
 	PRCONF_DBL(fps);
 	PRCONF_DBL(flushdelay);
