@@ -41,7 +41,13 @@ static const int pf2trans[PIXELFORMATS] =
 };
 
 
-FakerConfig *fconfig_getinstance(void) { return fconfig; }
+FakerConfig *fconfig_getinstance(void)
+{
+	#ifdef USEHELGRIND
+	ANNOTATE_BENIGN_RACE_SIZED(&fconfig, sizeof(FakerConfig *), );
+	#endif
+	return fconfig;
+}
 
 
 // This just wraps the VGLTrans class in order to demonstrate how to build a
@@ -57,14 +63,17 @@ void *RRTransInit(Display *dpy, Window win_, FakerConfig *fconfig_)
 	void *handle = NULL;
 	try
 	{
+		#ifdef USEHELGRIND
+		ANNOTATE_BENIGN_RACE_SIZED(&fconfig, sizeof(FakerConfig *), );
+		#endif
 		fconfig = fconfig_;
 		win = win_;
-		NEWCHECK(handle = (void *)(new VGLTrans()));
+		handle = (void *)(new VGLTrans());
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		handle = NULL;
 	}
 
@@ -85,10 +94,10 @@ int RRTransConnect(void *handle, char *receiverName, int port)
 		if(!vglconn) THROW("Invalid handle");
 		vglconn->connect(receiverName, port);
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
@@ -108,7 +117,7 @@ RRFrame *RRTransGetFrame(void *handle, int width, int height, int format,
 	{
 		VGLTrans *vglconn = (VGLTrans *)handle;
 		if(!vglconn) THROW("Invalid handle");
-		NEWCHECK(frame = new RRFrame);
+		frame = new RRFrame;
 		memset(frame, 0, sizeof(RRFrame));
 		int compress = fconfig->compress;
 		if(compress == RRCOMP_PROXY || compress == RRCOMP_RGB)
@@ -128,10 +137,10 @@ RRFrame *RRTransGetFrame(void *handle, int width, int height, int format,
 		frame->format = pf2trans[f->pf->id];
 		if(frame->format < 0) THROW("Unsupported pixel format");
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		delete frame;
 		frame = NULL;
 	}
@@ -153,10 +162,10 @@ int RRTransReady(void *handle)
 		if(!vglconn) THROW("Invalid handle");
 		ret = (int)vglconn->isReady();
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
@@ -177,10 +186,10 @@ int RRTransSynchronize(void *handle)
 		if(!vglconn) THROW("Invalid handle");
 		vglconn->synchronize();
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
@@ -208,10 +217,10 @@ int RRTransSendFrame(void *handle, RRFrame *frame, int sync)
 		vglconn->sendFrame(f);
 		delete frame;
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
@@ -232,10 +241,10 @@ int RRTransDestroy(void *handle)
 		if(!vglconn) THROW("Invalid handle");
 		delete vglconn;
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
