@@ -15,11 +15,13 @@
 #include <X11/Xlib.h>
 #include "rrtransport.h"
 #include "X11Trans.h"
+#ifdef USEHELGRIND
+	#include <valgrind/helgrind.h>
+#endif
 
 extern "C" void _vgl_disableFaker(void);
 extern "C" void _vgl_enableFaker(void);
 
-using namespace vglutil;
 using namespace vglcommon;
 using namespace vglserver;
 
@@ -37,7 +39,13 @@ static const int pf2trans[PIXELFORMATS] =
 };
 
 
-FakerConfig *fconfig_getinstance(void) { return fconfig; }
+FakerConfig *fconfig_getinstance(void)
+{
+	#ifdef USEHELGRIND
+	ANNOTATE_BENIGN_RACE_SIZED(&fconfig, sizeof(FakerConfig *), );
+	#endif
+	return fconfig;
+}
 
 
 // This just wraps the X11Trans class in order to demonstrate how to
@@ -53,15 +61,18 @@ void *RRTransInit(Display *dpy_, Window win_, FakerConfig *fconfig_)
 	void *handle = NULL;
 	try
 	{
+		#ifdef USEHELGRIND
+		ANNOTATE_BENIGN_RACE_SIZED(&fconfig, sizeof(FakerConfig *), );
+		#endif
 		fconfig = fconfig_;
 		dpy = dpy_;
 		win = win_;
-		NEWCHECK(handle = (void *)(new X11Trans()));
+		handle = (void *)(new X11Trans());
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		handle = NULL;
 	}
 
@@ -87,7 +98,7 @@ RRFrame *RRTransGetFrame(void *handle, int width, int height, int format,
 	{
 		X11Trans *trans = (X11Trans *)handle;
 		if(!trans) THROW("Invalid handle");
-		NEWCHECK(frame = new RRFrame);
+		frame = new RRFrame;
 		memset(frame, 0, sizeof(RRFrame));
 		FBXFrame *f = trans->getFrame(dpy, win, width, height);
 		f->flags |= FRAME_BOTTOMUP;
@@ -99,10 +110,10 @@ RRFrame *RRTransGetFrame(void *handle, int width, int height, int format,
 		frame->format = pf2trans[f->pf->id];
 		if(frame->format < 0) THROW("Unsupported pixel format");
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		delete frame;
 		frame = NULL;
 	}
@@ -124,10 +135,10 @@ int RRTransReady(void *handle)
 		if(!trans) THROW("Invalid handle");
 		ret = (int)trans->isReady();
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
@@ -148,10 +159,10 @@ int RRTransSynchronize(void *handle)
 		if(!trans) THROW("Invalid handle");
 		trans->synchronize();
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
@@ -176,10 +187,10 @@ int RRTransSendFrame(void *handle, RRFrame *frame, int sync)
 		trans->sendFrame(f, (bool)sync);
 		delete frame;
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
@@ -200,10 +211,10 @@ int RRTransDestroy(void *handle)
 		if(!trans) THROW("Invalid handle");
 		delete trans;
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", e.getMethod(),
-			e.getMessage());
+		snprintf(errStr, MAXSTR + 14, "Error in %s -- %s", GET_METHOD(e),
+			e.what());
 		ret = -1;
 	}
 
