@@ -2,7 +2,7 @@
  * Mesa 3-D graphics library
  *
  * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
- * Copyright (C) 2011-2012, 2014-2015, 2017-2019  D. R. Commander
+ * Copyright (C) 2011-2012, 2014-2015, 2017-2020  D. R. Commander
  *                                                All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -121,7 +121,7 @@ Fake_glXUseXFont(Font font, int first, int count, int listbase)
    unsigned int max_width, max_height, max_bm_width, max_bm_height;
    GLubyte *bm = NULL;
    int i, j, ngroups, groupsize, n;
-   VirtualWin *pbw;
+   vglfaker::VirtualWin *pbw;
    typedef struct {
       int bm_width, bm_height, width, height, valid;
       GLfloat x0, y0, dx, dy;
@@ -130,9 +130,9 @@ Fake_glXUseXFont(Font font, int first, int count, int listbase)
 
    try {
 
-   GLXDrawable draw = _glXGetCurrentDrawable();
+   GLXDrawable draw = VGLGetCurrentDrawable();
    if (!draw) return;
-   if (winhash.find(draw, pbw)) {
+   if ((pbw = winhash.find(NULL, draw)) != NULL) {
       /* Current drawable is a virtualized Window */
       ERRIFNOT(dpy = pbw->getX11Display());
       ERRIFNOT(win = pbw->getX11Drawable());
@@ -165,16 +165,16 @@ Fake_glXUseXFont(Font font, int first, int count, int listbase)
                                 InputOutput, v->visual,
                                 CWBorderPixel | CWColormap | CWEventMask,
                                 &swa)) == 0) {
-         XFree(v);
+         _XFree(v);
          THROW("Could not create temporary window for font rendering");
       }
-      XFree(v);
+      _XFree(v);
       newwin = true;
    }
 
    fs = XQueryFont(dpy, font);
    if (!fs) {
-      vglfaker::sendGLXError(X_GLXUseXFont, BadFont, true);
+      vglfaker::sendGLXError(dpy, X_GLXUseXFont, BadFont, true);
       return;
    }
 
@@ -185,7 +185,7 @@ Fake_glXUseXFont(Font font, int first, int count, int listbase)
          char *name = XGetAtomName(dpy, name_value);
 
          if (name) {
-            PRARGS(name);  XFree(name);
+            PRARGS(name);  _XFree(name);
          }
       }
    }
@@ -290,7 +290,7 @@ Fake_glXUseXFont(Font font, int first, int count, int listbase)
       }
 
       XFreeGC(dpy, gc);  gc = 0;
-      ERRIFNOT(image = XGetImage(dpy, pixmap, 0, 0, 8 * max_bm_width * n,
+      ERRIFNOT(image = _XGetImage(dpy, pixmap, 0, 0, 8 * max_bm_width * n,
                                   max_bm_height, 1, XYPixmap));
       XFreePixmap(dpy, pixmap);  pixmap = 0;
 
