@@ -263,9 +263,9 @@ void nativeWrite(bool useShm)
 		else fprintf(stderr, " (no errors)\n");
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n", e.getMessage());  retCode = -1;
+		fprintf(stderr, "%s\n", e.what());  retCode = -1;
 	}
 
 	pixelOffset = i - 1;
@@ -318,9 +318,9 @@ void nativeRead(bool useShm)
 		else fprintf(stderr, " (no errors)\n");
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n", e.getMessage());  retCode = -1;
+		fprintf(stderr, "%s\n", e.what());  retCode = -1;
 	}
 
 	fbx_term(&fb);
@@ -454,9 +454,9 @@ void nativeStress(bool useShm)
 				(1000000. * testTime));
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n", e.getMessage());
+		fprintf(stderr, "%s\n", e.what());
 	}
 
 	try
@@ -490,9 +490,9 @@ void nativeStress(bool useShm)
 				(1000000. * testTime));
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n", e.getMessage());  retCode = -1;
+		fprintf(stderr, "%s\n", e.what());  retCode = -1;
 	}
 
 }
@@ -889,9 +889,9 @@ int main(int argc, char **argv)
 		#endif
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n", e.getMessage());  retCode = -1;
+		fprintf(stderr, "%s\n", e.what());  retCode = -1;
 	}
 
 	return retCode;
