@@ -387,9 +387,9 @@ int writeTest(int format)
 		fprintf(stderr, "%s Mpixels/sec\n", sigFig(4, temps, avgmps));
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n", e.getMessage());  retval = -1;
+		fprintf(stderr, "%s\n", e.what());  retval = -1;
 	}
 
 	free(rgbaBuffer);
@@ -550,9 +550,9 @@ int readTest(int format)
 			sigFig(4, temps, readPixelsTime / rbtime * 100.0));
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n", e.getMessage());  retval = -1;
+		fprintf(stderr, "%s\n", e.what());  retval = -1;
 	}
 
 	free(rgbaBuffer);
@@ -829,9 +829,9 @@ int main(int argc, char **argv)
 		return 0;
 
 	}
-	catch(Error &e)
+	catch(std::exception &e)
 	{
-		fprintf(stderr, "%s\n", e.getMessage());
+		fprintf(stderr, "%s\n", e.what());
 	}
 
 	if(dpy)
